﻿/*
 * Author: Guadmaz <rockeurp@gmail.com>
 * Do not redistribute this without my authorization.
 */
using System;
using System.Drawing;
using System.Windows.Forms;
using GTA;
using GTA.Math;
using GTA.Native;
namespace SearchLight
{
    public class SearchLight : Script
    {
        public SearchLight()
        {
            Tick += OnTick;
            KeyDown += OnKeyDown;

            var settings = ScriptSettings.Load("scripts\\SearchLight.ini");
            if (settings != null)
            {
                _enableKey = settings.GetValue<Keys>("KEY_SETTINGS", "EnableSearchlight", Keys.NumPad0);
                _vehicleKey = settings.GetValue<Keys>("KEY_SETTINGS", "SearchVehicle", Keys.NumPad1);
                _pedKey = settings.GetValue<Keys>("KEY_SETTINGS", "SearchPerson", Keys.NumPad3);
                _debugKey = settings.GetValue<Keys>("KEY_SETTINGS", "DrawLine", Keys.Decimal);
                _manualKey = settings.GetValue<Keys>("KEY_SETTINGS", "ManualModeToggle", Keys.NumPad2);

                _manUp = settings.GetValue<Keys>("KEY_SETTINGS", "ManualUP", Keys.NumPad8);
                _manDown = settings.GetValue<Keys>("KEY_SETTINGS", "ManualDOWN", Keys.NumPad5);
                _manLeft = settings.GetValue<Keys>("KEY_SETTINGS", "ManualLEFT", Keys.NumPad4);
                _manRight = settings.GetValue<Keys>("KEY_SETTINGS", "ManualRIGHT", Keys.NumPad6);
            }
            else
            {
                _enableKey = Keys.NumPad0;
                _vehicleKey = Keys.NumPad1;
                _pedKey = Keys.NumPad3;
                _debugKey = Keys.Decimal;
                _manualKey = Keys.NumPad2;

                _manUp = Keys.NumPad8;
                _manDown = Keys.NumPad5;
                _manLeft = Keys.NumPad4;
                _manRight = Keys.NumPad6;
            }
        }

        private Entity _targetEntity;
        private bool _debugActive;
        private bool _lightActive;
        private bool _manualMode;

        private Keys _enableKey;
        private Keys _vehicleKey;
        private Keys _pedKey;
        private Keys _debugKey;
        private Keys _manualKey;

        private Keys _manUp;
        private Keys _manDown;
        private Keys _manLeft;
        private Keys _manRight;

        //Manual mode
        private float counterX = 1;
        private float counterY;

        void OnTick(object sender, EventArgs e)
        {
            Ped player = Game.Player.Character;

            if (_targetEntity != null && player.IsInVehicle() && _lightActive)
            {
                if (player.CurrentVehicle.Model == VehicleHash.Polmav && ((_targetEntity.Position - player.Position).Length() < 150f || _manualMode))
                {
                    Vector3 dest = _targetEntity.Position - player.Position - player.ForwardVector*3.4f;
                    Vector3 origin = player.CurrentVehicle.Position + player.ForwardVector*3.4f - new Vector3(0, 0, 1f);
                    if (_manualMode)
                    {
                        dest =
                            new Vector3(
                                (float)Math.Cos((double)player.CurrentVehicle.Heading * 0.0174532925f + counterX),
                                (float)Math.Sin((double)player.CurrentVehicle.Heading * 0.0174532925f + counterX),
                                counterY);
                    }
                    dest.Normalize();
                    if (_manualMode)
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, 100f, 10f, 1f, 5f, 1f, 0);
                    else
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, (_targetEntity.Position - player.Position).Length() + 20, 10f, 1f, 5f, 1f, 0);
                    if(_debugActive)
                        Function.Call(Hash.DRAW_LINE, origin.X, origin.Y, origin.Z, _targetEntity.Position.X, _targetEntity.Position.Y, _targetEntity.Position.Z, 255, 255, 255, 255);
                    Function.Call(Hash.DRAW_LIGHT_WITH_RANGE, origin.X, origin.Y, origin.Z, 255, 255, 255, 1f, 10f);
                }
                if (player.CurrentVehicle.Model == VehicleHash.Police3 && ((_targetEntity.Position - player.Position).Length() < 150f || _manualMode))
                {
                    Vector3 offset = new Vector3((float)Math.Cos((double)player.CurrentVehicle.Heading * 0.0174532925f) * -0.9f, (float)Math.Sin((double)player.CurrentVehicle.Heading * 0.0174532925f) * -0.9f, 0.5f);
                    Vector3 origin = player.CurrentVehicle.Position  + offset + player.ForwardVector*0.9f;
                    Vector3 dest = _targetEntity.Position - player.Position - offset - player.ForwardVector * 0.9f;
                    if (_manualMode)
                    {
                        dest =
                            new Vector3(
                                (float) Math.Cos((double) player.CurrentVehicle.Heading*0.0174532925f + counterX),
                                (float) Math.Sin((double) player.CurrentVehicle.Heading*0.0174532925f + counterX),
                                counterY);
                    }
                    dest.Normalize();
                    if(_manualMode)
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, 30f, 5f, 1f, 15f, 1f, 0);
                    else
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, (_targetEntity.Position - player.Position).Length() + 20, 5f, 1f, 15f, 1f, 0);
                    if (_debugActive)
                        Function.Call(Hash.DRAW_LINE, origin.X, origin.Y, origin.Z, _targetEntity.Position.X, _targetEntity.Position.Y, _targetEntity.Position.Z, 255, 255, 255, 255);
                    Function.Call(Hash.DRAW_LIGHT_WITH_RANGE, origin.X, origin.Y, origin.Z, 255, 255, 255, 0.2f, 10f);
                }
                if (player.CurrentVehicle.Model == VehicleHash.Sheriff2 && ((_targetEntity.Position - player.Position).Length() < 150f || _manualMode))
                {
                    Vector3 offset = new Vector3((float)Math.Cos((double)player.CurrentVehicle.Heading * 0.0174532925f)*-0.95f, (float)Math.Sin((double)player.CurrentVehicle.Heading * 0.0174532925f)*-0.95f, 0.9f);
                    Vector3 origin = player.CurrentVehicle.Position + offset + player.ForwardVector * 1.3f;
                    Vector3 dest = _targetEntity.Position - player.Position - offset - player.ForwardVector * 1.3f;
                    if (_manualMode)
                    {
                        dest =
                            new Vector3(
                                (float)Math.Cos((double)player.CurrentVehicle.Heading * 0.0174532925f + counterX),
                                (float)Math.Sin((double)player.CurrentVehicle.Heading * 0.0174532925f + counterX),
                                counterY);
                    }
                    dest.Normalize();
                    if (_manualMode)
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, 30f, 5f, 1f, 15f, 1f, 0);
                    else
                        Function.Call(Hash._0x5BCA583A583194DB, origin.X, origin.Y, origin.Z, dest.X, dest.Y, dest.Z, 255, 255, 255, (_targetEntity.Position - player.Position).Length() + 20, 5f, 1f, 15f, 1f, 0);
                    if (_debugActive)
                        Function.Call(Hash.DRAW_LINE, origin.X, origin.Y, origin.Z, _targetEntity.Position.X, _targetEntity.Position.Y, _targetEntity.Position.Z, 255, 255, 255, 255);
                    Function.Call(Hash.DRAW_LIGHT_WITH_RANGE, origin.X, origin.Y, origin.Z, 255, 255, 255, 0.2f, 10f);
                }
            }
        }
        private void OnKeyDown(object sender, KeyEventArgs e)
        {
            Ped player = Game.Player.Character;
            if (e.KeyCode == _enableKey)
            {
                _lightActive = !_lightActive;
                if (_lightActive && _targetEntity == null)
                {
                    _targetEntity = World.GetClosestVehicle(player.Position, 150f);
                }
            }
            else if (e.KeyCode == _vehicleKey)
                _targetEntity = World.GetClosestVehicle(player.Position + player.ForwardVector*10f, 150f);
            else if (e.KeyCode == _pedKey)
                _targetEntity = World.GetClosestPed(player.Position + player.ForwardVector * 10f, 150f);
            else if (e.KeyCode == _debugKey)
                _debugActive = !_debugActive;
            else if (e.KeyCode == _manualKey)
                _manualMode = !_manualMode;
            else if (e.KeyCode == _manUp && _manualMode && player.IsInVehicle())
                counterY += 0.1f;
            else if (e.KeyCode == _manDown && _manualMode && player.IsInVehicle())
                counterY -= 0.1f;
            else if (e.KeyCode == _manLeft && _manualMode && player.IsInVehicle())
                counterX += 0.1f;
            else if (e.KeyCode == _manRight && _manualMode && player.IsInVehicle())
                counterX -= 0.1f;
        }
    }
}
